# coding: utf-8


import pprint
import re  # noqa: F401

import six


class RelayerApiOrdersChannelSubscribeSchema(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        "type": "str",
        "channel": "str",
        "request_id": "str",
        "payload": "RelayerApiOrdersChannelSubscribePayloadSchema",
    }

    attribute_map = {
        "type": "type",
        "channel": "channel",
        "request_id": "requestId",
        "payload": "payload",
    }

    def __init__(
        self, type=None, channel=None, request_id=None, payload=None
    ):  # noqa: E501
        """RelayerApiOrdersChannelSubscribeSchema - a model defined in OpenAPI"""  # noqa: E501

        self._type = None
        self._channel = None
        self._request_id = None
        self._payload = None
        self.discriminator = None

        self.type = type
        self.channel = channel
        self.request_id = request_id
        if payload is not None:
            self.payload = payload

    @property
    def type(self):
        """Gets the type of this RelayerApiOrdersChannelSubscribeSchema.  # noqa: E501


        :return: The type of this RelayerApiOrdersChannelSubscribeSchema.  # noqa: E501
        :rtype: str
        """
        return self._type

    @type.setter
    def type(self, type):
        """Sets the type of this RelayerApiOrdersChannelSubscribeSchema.


        :param type: The type of this RelayerApiOrdersChannelSubscribeSchema.  # noqa: E501
        :type: str
        """
        if type is None:
            raise ValueError(
                "Invalid value for `type`, must not be `None`"
            )  # noqa: E501
        allowed_values = ["subscribe"]  # noqa: E501
        if type not in allowed_values:
            raise ValueError(
                "Invalid value for `type` ({0}), must be one of {1}".format(  # noqa: E501
                    type, allowed_values
                )
            )

        self._type = type

    @property
    def channel(self):
        """Gets the channel of this RelayerApiOrdersChannelSubscribeSchema.  # noqa: E501


        :return: The channel of this RelayerApiOrdersChannelSubscribeSchema.  # noqa: E501
        :rtype: str
        """
        return self._channel

    @channel.setter
    def channel(self, channel):
        """Sets the channel of this RelayerApiOrdersChannelSubscribeSchema.


        :param channel: The channel of this RelayerApiOrdersChannelSubscribeSchema.  # noqa: E501
        :type: str
        """
        if channel is None:
            raise ValueError(
                "Invalid value for `channel`, must not be `None`"
            )  # noqa: E501
        allowed_values = ["orders"]  # noqa: E501
        if channel not in allowed_values:
            raise ValueError(
                "Invalid value for `channel` ({0}), must be one of {1}".format(  # noqa: E501
                    channel, allowed_values
                )
            )

        self._channel = channel

    @property
    def request_id(self):
        """Gets the request_id of this RelayerApiOrdersChannelSubscribeSchema.  # noqa: E501


        :return: The request_id of this RelayerApiOrdersChannelSubscribeSchema.  # noqa: E501
        :rtype: str
        """
        return self._request_id

    @request_id.setter
    def request_id(self, request_id):
        """Sets the request_id of this RelayerApiOrdersChannelSubscribeSchema.


        :param request_id: The request_id of this RelayerApiOrdersChannelSubscribeSchema.  # noqa: E501
        :type: str
        """
        if request_id is None:
            raise ValueError(
                "Invalid value for `request_id`, must not be `None`"
            )  # noqa: E501

        self._request_id = request_id

    @property
    def payload(self):
        """Gets the payload of this RelayerApiOrdersChannelSubscribeSchema.  # noqa: E501


        :return: The payload of this RelayerApiOrdersChannelSubscribeSchema.  # noqa: E501
        :rtype: RelayerApiOrdersChannelSubscribePayloadSchema
        """
        return self._payload

    @payload.setter
    def payload(self, payload):
        """Sets the payload of this RelayerApiOrdersChannelSubscribeSchema.


        :param payload: The payload of this RelayerApiOrdersChannelSubscribeSchema.  # noqa: E501
        :type: RelayerApiOrdersChannelSubscribePayloadSchema
        """

        self._payload = payload

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(
                    map(
                        lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                        value,
                    )
                )
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(
                    map(
                        lambda item: (item[0], item[1].to_dict())
                        if hasattr(item[1], "to_dict")
                        else item,
                        value.items(),
                    )
                )
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, RelayerApiOrdersChannelSubscribeSchema):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8


import pprint
import re  # noqa: F401

import six


class RelayerApiOrdersChannelSubscribePayloadSchema(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        "maker_asset_proxy_id": "str",
        "taker_asset_proxy_id": "str",
        "network_id": "float",
        "maker_asset_address": "str",
        "taker_asset_address": "str",
        "maker_asset_data": "str",
        "taker_asset_data": "str",
        "trader_asset_data": "str",
    }

    attribute_map = {
        "maker_asset_proxy_id": "makerAssetProxyId",
        "taker_asset_proxy_id": "takerAssetProxyId",
        "network_id": "networkId",
        "maker_asset_address": "makerAssetAddress",
        "taker_asset_address": "takerAssetAddress",
        "maker_asset_data": "makerAssetData",
        "taker_asset_data": "takerAssetData",
        "trader_asset_data": "traderAssetData",
    }

    def __init__(
        self,
        maker_asset_proxy_id=None,
        taker_asset_proxy_id=None,
        network_id=None,
        maker_asset_address=None,
        taker_asset_address=None,
        maker_asset_data=None,
        taker_asset_data=None,
        trader_asset_data=None,
    ):  # noqa: E501
        """RelayerApiOrdersChannelSubscribePayloadSchema - a model defined in OpenAPI"""  # noqa: E501

        self._maker_asset_proxy_id = None
        self._taker_asset_proxy_id = None
        self._network_id = None
        self._maker_asset_address = None
        self._taker_asset_address = None
        self._maker_asset_data = None
        self._taker_asset_data = None
        self._trader_asset_data = None
        self.discriminator = None

        if maker_asset_proxy_id is not None:
            self.maker_asset_proxy_id = maker_asset_proxy_id
        if taker_asset_proxy_id is not None:
            self.taker_asset_proxy_id = taker_asset_proxy_id
        if network_id is not None:
            self.network_id = network_id
        if maker_asset_address is not None:
            self.maker_asset_address = maker_asset_address
        if taker_asset_address is not None:
            self.taker_asset_address = taker_asset_address
        if maker_asset_data is not None:
            self.maker_asset_data = maker_asset_data
        if taker_asset_data is not None:
            self.taker_asset_data = taker_asset_data
        if trader_asset_data is not None:
            self.trader_asset_data = trader_asset_data

    @property
    def maker_asset_proxy_id(self):
        """Gets the maker_asset_proxy_id of this RelayerApiOrdersChannelSubscribePayloadSchema.  # noqa: E501


        :return: The maker_asset_proxy_id of this RelayerApiOrdersChannelSubscribePayloadSchema.  # noqa: E501
        :rtype: str
        """
        return self._maker_asset_proxy_id

    @maker_asset_proxy_id.setter
    def maker_asset_proxy_id(self, maker_asset_proxy_id):
        """Sets the maker_asset_proxy_id of this RelayerApiOrdersChannelSubscribePayloadSchema.


        :param maker_asset_proxy_id: The maker_asset_proxy_id of this RelayerApiOrdersChannelSubscribePayloadSchema.  # noqa: E501
        :type: str
        """
        if maker_asset_proxy_id is not None and not re.search(
            r"^0x(([0-9a-f][0-9a-f])+)?$", maker_asset_proxy_id
        ):  # noqa: E501
            raise ValueError(
                r"Invalid value for `maker_asset_proxy_id`, must be a follow pattern or equal to `/^0x(([0-9a-f][0-9a-f])+)?$/`"
            )  # noqa: E501

        self._maker_asset_proxy_id = maker_asset_proxy_id

    @property
    def taker_asset_proxy_id(self):
        """Gets the taker_asset_proxy_id of this RelayerApiOrdersChannelSubscribePayloadSchema.  # noqa: E501


        :return: The taker_asset_proxy_id of this RelayerApiOrdersChannelSubscribePayloadSchema.  # noqa: E501
        :rtype: str
        """
        return self._taker_asset_proxy_id

    @taker_asset_proxy_id.setter
    def taker_asset_proxy_id(self, taker_asset_proxy_id):
        """Sets the taker_asset_proxy_id of this RelayerApiOrdersChannelSubscribePayloadSchema.


        :param taker_asset_proxy_id: The taker_asset_proxy_id of this RelayerApiOrdersChannelSubscribePayloadSchema.  # noqa: E501
        :type: str
        """
        if taker_asset_proxy_id is not None and not re.search(
            r"^0x(([0-9a-f][0-9a-f])+)?$", taker_asset_proxy_id
        ):  # noqa: E501
            raise ValueError(
                r"Invalid value for `taker_asset_proxy_id`, must be a follow pattern or equal to `/^0x(([0-9a-f][0-9a-f])+)?$/`"
            )  # noqa: E501

        self._taker_asset_proxy_id = taker_asset_proxy_id

    @property
    def network_id(self):
        """Gets the network_id of this RelayerApiOrdersChannelSubscribePayloadSchema.  # noqa: E501


        :return: The network_id of this RelayerApiOrdersChannelSubscribePayloadSchema.  # noqa: E501
        :rtype: float
        """
        return self._network_id

    @network_id.setter
    def network_id(self, network_id):
        """Sets the network_id of this RelayerApiOrdersChannelSubscribePayloadSchema.


        :param network_id: The network_id of this RelayerApiOrdersChannelSubscribePayloadSchema.  # noqa: E501
        :type: float
        """

        self._network_id = network_id

    @property
    def maker_asset_address(self):
        """Gets the maker_asset_address of this RelayerApiOrdersChannelSubscribePayloadSchema.  # noqa: E501


        :return: The maker_asset_address of this RelayerApiOrdersChannelSubscribePayloadSchema.  # noqa: E501
        :rtype: str
        """
        return self._maker_asset_address

    @maker_asset_address.setter
    def maker_asset_address(self, maker_asset_address):
        """Sets the maker_asset_address of this RelayerApiOrdersChannelSubscribePayloadSchema.


        :param maker_asset_address: The maker_asset_address of this RelayerApiOrdersChannelSubscribePayloadSchema.  # noqa: E501
        :type: str
        """
        if maker_asset_address is not None and not re.search(
            r"^0x[0-9a-f]{40}$", maker_asset_address
        ):  # noqa: E501
            raise ValueError(
                r"Invalid value for `maker_asset_address`, must be a follow pattern or equal to `/^0x[0-9a-f]{40}$/`"
            )  # noqa: E501

        self._maker_asset_address = maker_asset_address

    @property
    def taker_asset_address(self):
        """Gets the taker_asset_address of this RelayerApiOrdersChannelSubscribePayloadSchema.  # noqa: E501


        :return: The taker_asset_address of this RelayerApiOrdersChannelSubscribePayloadSchema.  # noqa: E501
        :rtype: str
        """
        return self._taker_asset_address

    @taker_asset_address.setter
    def taker_asset_address(self, taker_asset_address):
        """Sets the taker_asset_address of this RelayerApiOrdersChannelSubscribePayloadSchema.


        :param taker_asset_address: The taker_asset_address of this RelayerApiOrdersChannelSubscribePayloadSchema.  # noqa: E501
        :type: str
        """
        if taker_asset_address is not None and not re.search(
            r"^0x[0-9a-f]{40}$", taker_asset_address
        ):  # noqa: E501
            raise ValueError(
                r"Invalid value for `taker_asset_address`, must be a follow pattern or equal to `/^0x[0-9a-f]{40}$/`"
            )  # noqa: E501

        self._taker_asset_address = taker_asset_address

    @property
    def maker_asset_data(self):
        """Gets the maker_asset_data of this RelayerApiOrdersChannelSubscribePayloadSchema.  # noqa: E501


        :return: The maker_asset_data of this RelayerApiOrdersChannelSubscribePayloadSchema.  # noqa: E501
        :rtype: str
        """
        return self._maker_asset_data

    @maker_asset_data.setter
    def maker_asset_data(self, maker_asset_data):
        """Sets the maker_asset_data of this RelayerApiOrdersChannelSubscribePayloadSchema.


        :param maker_asset_data: The maker_asset_data of this RelayerApiOrdersChannelSubscribePayloadSchema.  # noqa: E501
        :type: str
        """
        if maker_asset_data is not None and not re.search(
            r"^0x(([0-9a-f][0-9a-f])+)?$", maker_asset_data
        ):  # noqa: E501
            raise ValueError(
                r"Invalid value for `maker_asset_data`, must be a follow pattern or equal to `/^0x(([0-9a-f][0-9a-f])+)?$/`"
            )  # noqa: E501

        self._maker_asset_data = maker_asset_data

    @property
    def taker_asset_data(self):
        """Gets the taker_asset_data of this RelayerApiOrdersChannelSubscribePayloadSchema.  # noqa: E501


        :return: The taker_asset_data of this RelayerApiOrdersChannelSubscribePayloadSchema.  # noqa: E501
        :rtype: str
        """
        return self._taker_asset_data

    @taker_asset_data.setter
    def taker_asset_data(self, taker_asset_data):
        """Sets the taker_asset_data of this RelayerApiOrdersChannelSubscribePayloadSchema.


        :param taker_asset_data: The taker_asset_data of this RelayerApiOrdersChannelSubscribePayloadSchema.  # noqa: E501
        :type: str
        """
        if taker_asset_data is not None and not re.search(
            r"^0x(([0-9a-f][0-9a-f])+)?$", taker_asset_data
        ):  # noqa: E501
            raise ValueError(
                r"Invalid value for `taker_asset_data`, must be a follow pattern or equal to `/^0x(([0-9a-f][0-9a-f])+)?$/`"
            )  # noqa: E501

        self._taker_asset_data = taker_asset_data

    @property
    def trader_asset_data(self):
        """Gets the trader_asset_data of this RelayerApiOrdersChannelSubscribePayloadSchema.  # noqa: E501


        :return: The trader_asset_data of this RelayerApiOrdersChannelSubscribePayloadSchema.  # noqa: E501
        :rtype: str
        """
        return self._trader_asset_data

    @trader_asset_data.setter
    def trader_asset_data(self, trader_asset_data):
        """Sets the trader_asset_data of this RelayerApiOrdersChannelSubscribePayloadSchema.


        :param trader_asset_data: The trader_asset_data of this RelayerApiOrdersChannelSubscribePayloadSchema.  # noqa: E501
        :type: str
        """
        if trader_asset_data is not None and not re.search(
            r"^0x(([0-9a-f][0-9a-f])+)?$", trader_asset_data
        ):  # noqa: E501
            raise ValueError(
                r"Invalid value for `trader_asset_data`, must be a follow pattern or equal to `/^0x(([0-9a-f][0-9a-f])+)?$/`"
            )  # noqa: E501

        self._trader_asset_data = trader_asset_data

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(
                    map(
                        lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                        value,
                    )
                )
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(
                    map(
                        lambda item: (item[0], item[1].to_dict())
                        if hasattr(item[1], "to_dict")
                        else item,
                        value.items(),
                    )
                )
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(
            other, RelayerApiOrdersChannelSubscribePayloadSchema
        ):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8


import pprint
import re  # noqa: F401

import six


class RelayerApiOrderConfigResponseSchema(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        "maker_fee": "str",
        "taker_fee": "str",
        "fee_recipient_address": "str",
        "sender_address": "str",
    }

    attribute_map = {
        "maker_fee": "makerFee",
        "taker_fee": "takerFee",
        "fee_recipient_address": "feeRecipientAddress",
        "sender_address": "senderAddress",
    }

    def __init__(
        self,
        maker_fee=None,
        taker_fee=None,
        fee_recipient_address=None,
        sender_address=None,
    ):  # noqa: E501
        """RelayerApiOrderConfigResponseSchema - a model defined in OpenAPI"""  # noqa: E501

        self._maker_fee = None
        self._taker_fee = None
        self._fee_recipient_address = None
        self._sender_address = None
        self.discriminator = None

        self.maker_fee = maker_fee
        self.taker_fee = taker_fee
        self.fee_recipient_address = fee_recipient_address
        self.sender_address = sender_address

    @property
    def maker_fee(self):
        """Gets the maker_fee of this RelayerApiOrderConfigResponseSchema.  # noqa: E501


        :return: The maker_fee of this RelayerApiOrderConfigResponseSchema.  # noqa: E501
        :rtype: str
        """
        return self._maker_fee

    @maker_fee.setter
    def maker_fee(self, maker_fee):
        """Sets the maker_fee of this RelayerApiOrderConfigResponseSchema.


        :param maker_fee: The maker_fee of this RelayerApiOrderConfigResponseSchema.  # noqa: E501
        :type: str
        """
        if maker_fee is None:
            raise ValueError(
                "Invalid value for `maker_fee`, must not be `None`"
            )  # noqa: E501
        if maker_fee is not None and not re.search(
            r"^\\d+$", maker_fee
        ):  # noqa: E501
            raise ValueError(
                r"Invalid value for `maker_fee`, must be a follow pattern or equal to `/^\\d+$/`"
            )  # noqa: E501

        self._maker_fee = maker_fee

    @property
    def taker_fee(self):
        """Gets the taker_fee of this RelayerApiOrderConfigResponseSchema.  # noqa: E501


        :return: The taker_fee of this RelayerApiOrderConfigResponseSchema.  # noqa: E501
        :rtype: str
        """
        return self._taker_fee

    @taker_fee.setter
    def taker_fee(self, taker_fee):
        """Sets the taker_fee of this RelayerApiOrderConfigResponseSchema.


        :param taker_fee: The taker_fee of this RelayerApiOrderConfigResponseSchema.  # noqa: E501
        :type: str
        """
        if taker_fee is None:
            raise ValueError(
                "Invalid value for `taker_fee`, must not be `None`"
            )  # noqa: E501
        if taker_fee is not None and not re.search(
            r"^\\d+$", taker_fee
        ):  # noqa: E501
            raise ValueError(
                r"Invalid value for `taker_fee`, must be a follow pattern or equal to `/^\\d+$/`"
            )  # noqa: E501

        self._taker_fee = taker_fee

    @property
    def fee_recipient_address(self):
        """Gets the fee_recipient_address of this RelayerApiOrderConfigResponseSchema.  # noqa: E501


        :return: The fee_recipient_address of this RelayerApiOrderConfigResponseSchema.  # noqa: E501
        :rtype: str
        """
        return self._fee_recipient_address

    @fee_recipient_address.setter
    def fee_recipient_address(self, fee_recipient_address):
        """Sets the fee_recipient_address of this RelayerApiOrderConfigResponseSchema.


        :param fee_recipient_address: The fee_recipient_address of this RelayerApiOrderConfigResponseSchema.  # noqa: E501
        :type: str
        """
        if fee_recipient_address is None:
            raise ValueError(
                "Invalid value for `fee_recipient_address`, must not be `None`"
            )  # noqa: E501
        if fee_recipient_address is not None and not re.search(
            r"^0x[0-9a-f]{40}$", fee_recipient_address
        ):  # noqa: E501
            raise ValueError(
                r"Invalid value for `fee_recipient_address`, must be a follow pattern or equal to `/^0x[0-9a-f]{40}$/`"
            )  # noqa: E501

        self._fee_recipient_address = fee_recipient_address

    @property
    def sender_address(self):
        """Gets the sender_address of this RelayerApiOrderConfigResponseSchema.  # noqa: E501


        :return: The sender_address of this RelayerApiOrderConfigResponseSchema.  # noqa: E501
        :rtype: str
        """
        return self._sender_address

    @sender_address.setter
    def sender_address(self, sender_address):
        """Sets the sender_address of this RelayerApiOrderConfigResponseSchema.


        :param sender_address: The sender_address of this RelayerApiOrderConfigResponseSchema.  # noqa: E501
        :type: str
        """
        if sender_address is None:
            raise ValueError(
                "Invalid value for `sender_address`, must not be `None`"
            )  # noqa: E501
        if sender_address is not None and not re.search(
            r"^0x[0-9a-f]{40}$", sender_address
        ):  # noqa: E501
            raise ValueError(
                r"Invalid value for `sender_address`, must be a follow pattern or equal to `/^0x[0-9a-f]{40}$/`"
            )  # noqa: E501

        self._sender_address = sender_address

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(
                    map(
                        lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                        value,
                    )
                )
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(
                    map(
                        lambda item: (item[0], item[1].to_dict())
                        if hasattr(item[1], "to_dict")
                        else item,
                        value.items(),
                    )
                )
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, RelayerApiOrderConfigResponseSchema):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

# flake8: noqa

from __future__ import absolute_import

# import models into model package
from zero_ex.sra_client.models.order_schema import OrderSchema
from zero_ex.sra_client.models.paginated_collection_schema import (
    PaginatedCollectionSchema,
)
from zero_ex.sra_client.models.relayer_api_asset_data_pairs_response_schema import (
    RelayerApiAssetDataPairsResponseSchema,
)
from zero_ex.sra_client.models.relayer_api_asset_data_trade_info_schema import (
    RelayerApiAssetDataTradeInfoSchema,
)
from zero_ex.sra_client.models.relayer_api_error_response_schema import (
    RelayerApiErrorResponseSchema,
)
from zero_ex.sra_client.models.relayer_api_error_response_schema_validation_errors import (
    RelayerApiErrorResponseSchemaValidationErrors,
)
from zero_ex.sra_client.models.relayer_api_fee_recipients_response_schema import (
    RelayerApiFeeRecipientsResponseSchema,
)
from zero_ex.sra_client.models.relayer_api_order_config_payload_schema import (
    RelayerApiOrderConfigPayloadSchema,
)
from zero_ex.sra_client.models.relayer_api_order_config_response_schema import (
    RelayerApiOrderConfigResponseSchema,
)
from zero_ex.sra_client.models.relayer_api_order_schema import RelayerApiOrderSchema
from zero_ex.sra_client.models.relayer_api_orderbook_response_schema import (
    RelayerApiOrderbookResponseSchema,
)
from zero_ex.sra_client.models.relayer_api_orders_channel_subscribe_payload_schema import (
    RelayerApiOrdersChannelSubscribePayloadSchema,
)
from zero_ex.sra_client.models.relayer_api_orders_channel_subscribe_schema import (
    RelayerApiOrdersChannelSubscribeSchema,
)
from zero_ex.sra_client.models.relayer_api_orders_channel_update_schema import (
    RelayerApiOrdersChannelUpdateSchema,
)
from zero_ex.sra_client.models.relayer_api_orders_response_schema import (
    RelayerApiOrdersResponseSchema,
)
from zero_ex.sra_client.models.signed_order_schema import SignedOrderSchema



# coding: utf-8


from __future__ import absolute_import

import re  # noqa: F401

# python 2 and python 3 compatibility library
import six

from zero_ex.sra_client.api_client import ApiClient
from zero_ex.sra_client.models.relayer_api_order_config_payload_schema import (
    RelayerApiOrderConfigPayloadSchema,
)


class DefaultApi(object):
    """Default API for SRA compliant 0x relayers."""

    # NOTE: This class is auto generated by OpenAPI Generator
    # Ref: https://openapi-generator.tech

    # Do not edit the class manually.

    def __init__(self, api_client=None):
        if api_client is None:
            api_client = ApiClient()
        self.api_client = api_client

    def get_asset_pairs(self, **kwargs):
        """get_asset_pairs

        Retrieves a list of available asset pairs and the information
        required to trade them (in any order). Setting only `assetDataA` or
        `assetDataB` returns pairs filtered by that asset only.
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass `async_req` = **True**

        >>> thread = api.get_asset_pairs(async_req=True) # doctest: +SKIP
        >>> result = thread.get() # doctest: +SKIP

        :param bool async_req: Whether request should be asynchronous.
        :param str asset_data_a: The assetData value for the first asset in the pair.
        :param str asset_data_b: The assetData value for the second asset in the pair.
        :param int network_id: The id of the Ethereum network
        :param int page: The number of the page to request in the collection.
        :param int per_page: The number of records to return per page.

        :return: :class:`RelayerApiAssetDataPairsResponseSchema`.
            If the method is called asynchronously returns the request thread.
        """
        kwargs["_return_http_data_only"] = True
        if kwargs.get("async_req"):
            return self.get_asset_pairs_with_http_info(**kwargs)
        else:
            (data) = self.get_asset_pairs_with_http_info(**kwargs)
            return data

    def get_asset_pairs_with_http_info(self, **kwargs):
        """get_asset_pairs

        Retrieves a list of available asset pairs and the information
        required to trade them (in any order). Setting only `assetDataA` or
        `assetDataB` returns pairs filtered by that asset only.
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass `async_req` = **True**

        >>> thread = api.get_asset_pairs_with_http_info(async_req=True) # doctest: +SKIP
        >>> result = thread.get() # doctest: +SKIP

        :param bool async_req: Whether request should be asynchronous.
        :param str asset_data_a: The assetData value for the first asset in the pair.
        :param str asset_data_b: The assetData value for the second asset in the pair.
        :param int network_id: The id of the Ethereum network
        :param int page: The number of the page to request in the collection.
        :param int per_page: The number of records to return per page.

        :return: A tuple consisting of a
            :class:`RelayerApiAssetDataPairsResponseSchema`, an HTTP status
            code integer, and a collection of HTTP headers.  If the method is
            called asynchronously returns the request thread.
        """

        local_var_params = locals()

        all_params = [
            "asset_data_a",
            "asset_data_b",
            "network_id",
            "page",
            "per_page",
        ]
        all_params.append("async_req")
        all_params.append("_return_http_data_only")
        all_params.append("_preload_content")
        all_params.append("_request_timeout")

        for key, val in six.iteritems(local_var_params["kwargs"]):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method get_asset_pairs" % key
                )
            local_var_params[key] = val
        del local_var_params["kwargs"]

        collection_formats = {}

        path_params = {}

        query_params = []
        if "asset_data_a" in local_var_params:
            query_params.append(
                ("assetDataA", local_var_params["asset_data_a"])
            )
        if "asset_data_b" in local_var_params:
            query_params.append(
                ("assetDataB", local_var_params["asset_data_b"])
            )
        if "network_id" in local_var_params:
            query_params.append(("networkId", local_var_params["network_id"]))
        if "page" in local_var_params:
            query_params.append(("page", local_var_params["page"]))
        if "per_page" in local_var_params:
            query_params.append(("perPage", local_var_params["per_page"]))

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params["Accept"] = self.api_client.select_header_accept(
            ["application/json"]
        )

        # Authentication setting
        auth_settings = []

        return self.api_client.call_api(
            "/v3/asset_pairs",
            "GET",
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type="RelayerApiAssetDataPairsResponseSchema",
            auth_settings=auth_settings,
            async_req=local_var_params.get("async_req"),
            _return_http_data_only=local_var_params.get(
                "_return_http_data_only"
            ),
            _preload_content=local_var_params.get("_preload_content", True),
            _request_timeout=local_var_params.get("_request_timeout"),
            collection_formats=collection_formats,
        )

    def get_fee_recipients(self, **kwargs):
        """get_fee_recipients

        Retrieves a collection of all fee recipient addresses for a relayer.
        This endpoint should be paginated.
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass `async_req` = **True**

        >>> thread = api.get_fee_recipients(async_req=True) # doctest: +SKIP
        >>> result = thread.get() # doctest: +SKIP

        :param bool async_req: Whether request should be asynchronous.
        :param int network_id: The id of the Ethereum network
        :param int page: The number of the page to request in the collection.
        :param int per_page: The number of records to return per page.

        :return: :class:`RelayerApiFeeRecipientsResponseSchema`.
            If the method is called asynchronously, returns the request thread.
        """
        kwargs["_return_http_data_only"] = True
        if kwargs.get("async_req"):
            return self.get_fee_recipients_with_http_info(**kwargs)
        else:
            (data) = self.get_fee_recipients_with_http_info(**kwargs)
            return data

    def get_fee_recipients_with_http_info(self, **kwargs):
        """get_fee_recipients

        Retrieves a collection of all fee recipient addresses for a relayer.
        This endpoint should be paginated.
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass `async_req` = **True**

        >>> thread = api.get_fee_recipients_with_http_info(async_req=True) # doctest: +SKIP
        >>> result = thread.get() # doctest: +SKIP

        :param bool async_req: Whether request should be asynchronous.
        :param int network_id: The id of the Ethereum network
        :param int page: The number of the page to request in the collection.
        :param int per_page: The number of records to return per page.

        :return: A tuple consisting of a
            :class:`RelayerApiFeeRecipientsResponseSchema`, an HTTP status
            code integer, and a collection of HTTP headers.  If the method is
            called asynchronously returns the request thread.
        """

        local_var_params = locals()

        all_params = ["network_id", "page", "per_page"]
        all_params.append("async_req")
        all_params.append("_return_http_data_only")
        all_params.append("_preload_content")
        all_params.append("_request_timeout")

        for key, val in six.iteritems(local_var_params["kwargs"]):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method get_fee_recipients" % key
                )
            local_var_params[key] = val
        del local_var_params["kwargs"]

        collection_formats = {}

        path_params = {}

        query_params = []
        if "network_id" in local_var_params:
            query_params.append(("networkId", local_var_params["network_id"]))
        if "page" in local_var_params:
            query_params.append(("page", local_var_params["page"]))
        if "per_page" in local_var_params:
            query_params.append(("perPage", local_var_params["per_page"]))

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params["Accept"] = self.api_client.select_header_accept(
            ["application/json"]
        )

        # Authentication setting
        auth_settings = []

        return self.api_client.call_api(
            "/v3/fee_recipients",
            "GET",
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type="RelayerApiFeeRecipientsResponseSchema",
            auth_settings=auth_settings,
            async_req=local_var_params.get("async_req"),
            _return_http_data_only=local_var_params.get(
                "_return_http_data_only"
            ),
            _preload_content=local_var_params.get("_preload_content", True),
            _request_timeout=local_var_params.get("_request_timeout"),
            collection_formats=collection_formats,
        )

    def get_order(self, order_hash, **kwargs):
        """get_order

        Retrieves the 0x order with meta info that is associated with the hash.
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass `async_req` = **True**

        >>> thread = api.get_order(async_req=True) # doctest: +SKIP
        >>> result = thread.get() # doctest: +SKIP

        :param bool async_req: Whether request should be asynchronous.
        :param str order_hash: The hash of the desired 0x order. (required)
        :param int network_id: The id of the Ethereum network

        :return: :class:`RelayerApiOrderSchema`.
            If the method is called asynchronously, returns the request thread.
        """
        kwargs["_return_http_data_only"] = True
        if kwargs.get("async_req"):
            return self.get_order_with_http_info(order_hash, **kwargs)
        else:
            (data) = self.get_order_with_http_info(order_hash, **kwargs)
            return data

    def get_order_with_http_info(self, order_hash, **kwargs):
        """get_order

        Retrieves the 0x order with meta info that is associated with the hash.
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass `async_req` = **True**

        >>> thread = api.get_order_with_http_info(async_req=True) # doctest: +SKIP
        >>> result = thread.get() # doctest: +SKIP

        :param bool async_req: Whether request should be asynchronous.
        :param str order_hash: The hash of the desired 0x order. (required)
        :param int network_id: The id of the Ethereum network

        :return: A tuple consisting of a
            :class:`RelayerApiOrderSchema`, an HTTP status code integer, and a
            collection of HTTP headers.  If the method is called
            asynchronously returns the request thread.
        """

        local_var_params = locals()

        all_params = ["order_hash", "network_id"]
        all_params.append("async_req")
        all_params.append("_return_http_data_only")
        all_params.append("_preload_content")
        all_params.append("_request_timeout")

        for key, val in six.iteritems(local_var_params["kwargs"]):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method get_order" % key
                )
            local_var_params[key] = val
        del local_var_params["kwargs"]
        # verify the required parameter 'order_hash' is set
        if (
            "order_hash" not in local_var_params
            or local_var_params["order_hash"] is None
        ):
            raise ValueError(
                "Missing the required parameter `order_hash` when calling `get_order`"
            )

        collection_formats = {}

        path_params = {}
        if "order_hash" in local_var_params:
            path_params["orderHash"] = local_var_params["order_hash"]

        query_params = []
        if "network_id" in local_var_params:
            query_params.append(("networkId", local_var_params["network_id"]))

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params["Accept"] = self.api_client.select_header_accept(
            ["application/json"]
        )

        # Authentication setting
        auth_settings = []

        return self.api_client.call_api(
            "/v3/order/{orderHash}",
            "GET",
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type="RelayerApiOrderSchema",
            auth_settings=auth_settings,
            async_req=local_var_params.get("async_req"),
            _return_http_data_only=local_var_params.get(
                "_return_http_data_only"
            ),
            _preload_content=local_var_params.get("_preload_content", True),
            _request_timeout=local_var_params.get("_request_timeout"),
            collection_formats=collection_formats,
        )

    def get_order_config(self, **kwargs):
        """get_order_config

        Relayers have full discretion over the orders that they are willing
        to host on their orderbooks (e.g what fees they charge, etc...). In
        order for traders to discover their requirements programmatically,
        they can send an incomplete order to this endpoint and receive the
        missing fields, specifc to that order. This gives relayers a large
        amount of flexibility to tailor fees to unique traders, trading pairs
        and volume amounts. Submit a partial order and receive information
        required to complete the order: `senderAddress`,
        `feeRecipientAddress`, `makerFee`, `takerFee`.
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass `async_req` = **True**

        >>> thread = api.get_order_config(async_req=True) # doctest: +SKIP
        >>> result = thread.get() # doctest: +SKIP

        :param bool async_req: Whether request should be asynchronous.
        :param int network_id: The id of the Ethereum network
        :param relayer_api_order_config_payload_schema: instance of
            :class:`RelayerApiOrderConfigPayloadSchema`. The fields of a 0x
            order the relayer may want to decide what configuration to send
            back.

        :return: :class:`RelayerApiOrderConfigResponseSchema`.
            If the method is called asynchronously returns the request thread.
        """
        kwargs["_return_http_data_only"] = True
        if kwargs.get("async_req"):
            return self.get_order_config_with_http_info(**kwargs)
        else:
            (data) = self.get_order_config_with_http_info(**kwargs)
            return data

    def get_order_config_with_http_info(self, **kwargs):
        """get_order_config

        Relayers have full discretion over the orders that they are willing
        to host on their orderbooks (e.g what fees they charge, etc...). In
        order for traders to discover their requirements programmatically,
        they can send an incomplete order to this endpoint and receive the
        missing fields, specifc to that order. This gives relayers a large
        amount of flexibility to tailor fees to unique traders, trading pairs
        and volume amounts. Submit a partial order and receive information
        required to complete the order: `senderAddress`,
        `feeRecipientAddress`, `makerFee`, `takerFee`.
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass `async_req` = **True**

        >>> thread = api.get_order_config_with_http_info(async_req=True) # doctest: +SKIP
        >>> result = thread.get() # doctest: +SKIP

        :param bool async_req: Whether request should be asynchronous.
        :param int network_id: The id of the Ethereum network
        :param relayer_api_order_config_payload_schema: instance of
            :class: `RelayerApiOrderConfigPayloadSchema`. The fields of a 0x
            order the relayer may want to decide what configuration to send
            back.

        :return: A tuple consisting of a
            :class:`RelayerApiOrderConfigResponseSchema`, an HTTP status code
            integer, and a collection of HTTP headers.  If the method is
            called asynchronously returns the request thread.
        """

        local_var_params = locals()

        all_params = ["network_id", "relayer_api_order_config_payload_schema"]
        all_params.append("async_req")
        all_params.append("_return_http_data_only")
        all_params.append("_preload_content")
        all_params.append("_request_timeout")

        for key, val in six.iteritems(local_var_params["kwargs"]):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method get_order_config" % key
                )
            local_var_params[key] = val
        del local_var_params["kwargs"]

        collection_formats = {}

        path_params = {}

        query_params = []
        if "network_id" in local_var_params:
            query_params.append(("networkId", local_var_params["network_id"]))

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        if "relayer_api_order_config_payload_schema" in local_var_params:
            body_params = local_var_params[
                "relayer_api_order_config_payload_schema"
            ]
        # HTTP header `Accept`
        header_params["Accept"] = self.api_client.select_header_accept(
            ["application/json"]
        )

        # HTTP header `Content-Type`
        header_params[
            "Content-Type"
        ] = self.api_client.select_header_content_type(["application/json"])

        # Authentication setting
        auth_settings = []

        return self.api_client.call_api(
            "/v3/order_config",
            "POST",
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type="RelayerApiOrderConfigResponseSchema",
            auth_settings=auth_settings,
            async_req=local_var_params.get("async_req"),
            _return_http_data_only=local_var_params.get(
                "_return_http_data_only"
            ),
            _preload_content=local_var_params.get("_preload_content", True),
            _request_timeout=local_var_params.get("_request_timeout"),
            collection_formats=collection_formats,
        )

    def get_orderbook(self, base_asset_data, quote_asset_data, **kwargs):
        """get_orderbook

        Retrieves the orderbook for a given asset pair. This endpoint should
        be paginated. Bids will be sorted in
        descending order by price, and asks will be sorted in ascending order
        by price. Within the price sorted orders, the orders are further
        sorted by **taker fee price** which is defined as the **takerFee**
        divided by **takerTokenAmount**. After **taker fee price**, orders are
        to be sorted by expiration in ascending order. The way pagination
        works for this endpoint is that the **page** and **perPage** query
        params apply to both `bids` and `asks` collections, and if `page` *
        `perPage` = `total` for a certain collection, the `records` for that
        collection should just be empty.
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass `async_req` = **True**

        >>> thread = api.get_orderbook(async_req=True) # doctest: +SKIP
        >>> result = thread.get() # doctest: +SKIP

        :param bool async_req: Whether request should be asynchronous.
        :param str base_asset_data: assetData (makerAssetData or
            takerAssetData) designated as the base currency in the
            `currency pair calculation
            <https://en.wikipedia.org/wiki/Currency_pair>`__
            of price. (required)
        :param str quote_asset_data: assetData (makerAssetData or
            takerAssetData) designated as the quote currency in the currency
            pair calculation of price. (required)
        :param int network_id: The id of the Ethereum network
        :param int page: The number of the page to request in the collection.
        :param int per_page: The number of records to return per page.

        :return: :class:`RelayerApiOrderbookResponseSchema`.
            If the method is called asynchronously, returns the request thread.
        """
        kwargs["_return_http_data_only"] = True
        if kwargs.get("async_req"):
            return self.get_orderbook_with_http_info(
                base_asset_data, quote_asset_data, **kwargs
            )
        else:
            (data) = self.get_orderbook_with_http_info(
                base_asset_data, quote_asset_data, **kwargs
            )
            return data

    def get_orderbook_with_http_info(
        self, base_asset_data, quote_asset_data, **kwargs
    ):
        """get_orderbook

        Retrieves the orderbook for a given asset pair. This endpoint should
        be paginated. Bids will be sorted in
        descending order by price, and asks will be sorted in ascending order
        by price. Within the price sorted orders, the orders are further
        sorted by **taker fee price** which is defined as the **takerFee**
        divided by **takerTokenAmount**. After **taker fee price**, orders are
        to be sorted by expiration in ascending order. The way pagination
        works for this endpoint is that the **page** and **perPage** query
        params apply to both `bids` and `asks` collections, and if `page` *
        `perPage` = `total` for a certain collection, the `records` for that
        collection should just be empty.
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass `async_req` = **True**

        >>> thread = api.get_orderbook_with_http_info(async_req=True) # doctest: +SKIP
        >>> result = thread.get() # doctest: +SKIP

        :param bool async_req: Whether request should be asynchronous.
        :param str base_asset_data: assetData (makerAssetData or
            takerAssetData) designated as the base currency in the
            `currency pair calculation
            <https://en.wikipedia.org/wiki/Currency_pair>`__
            of price. (required)
        :param str quote_asset_data: assetData (makerAssetData or
            takerAssetData) designated as the quote currency in the currency
            pair calculation of price. (required)
        :param int network_id: The id of the Ethereum network
        :param int page: The number of the page to request in the collection.
        :param int per_page: The number of records to return per page.

        :return: A tuple consisting of a
            :class:`RelayerApiOrderbookResponseSchema`, an HTTP status code
            integer, and a collection of HTTP headers.  If the method is
            called asynchronously returns the request thread.
        """

        local_var_params = locals()

        all_params = [
            "base_asset_data",
            "quote_asset_data",
            "network_id",
            "page",
            "per_page",
        ]
        all_params.append("async_req")
        all_params.append("_return_http_data_only")
        all_params.append("_preload_content")
        all_params.append("_request_timeout")

        for key, val in six.iteritems(local_var_params["kwargs"]):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method get_orderbook" % key
                )
            local_var_params[key] = val
        del local_var_params["kwargs"]
        # verify the required parameter 'base_asset_data' is set
        if (
            "base_asset_data" not in local_var_params
            or local_var_params["base_asset_data"] is None
        ):
            raise ValueError(
                "Missing the required parameter `base_asset_data`"
                "when calling `get_orderbook`"
            )
        # verify the required parameter 'quote_asset_data' is set
        if (
            "quote_asset_data" not in local_var_params
            or local_var_params["quote_asset_data"] is None
        ):
            raise ValueError(
                "Missing the required parameter `quote_asset_data`"
                " when calling `get_orderbook`"
            )

        collection_formats = {}

        path_params = {}

        query_params = []
        if "base_asset_data" in local_var_params:
            query_params.append(
                ("baseAssetData", local_var_params["base_asset_data"])
            )
        if "quote_asset_data" in local_var_params:
            query_params.append(
                ("quoteAssetData", local_var_params["quote_asset_data"])
            )
        if "network_id" in local_var_params:
            query_params.append(("networkId", local_var_params["network_id"]))
        if "page" in local_var_params:
            query_params.append(("page", local_var_params["page"]))
        if "per_page" in local_var_params:
            query_params.append(("perPage", local_var_params["per_page"]))

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params["Accept"] = self.api_client.select_header_accept(
            ["application/json"]
        )

        # Authentication setting
        auth_settings = []

        return self.api_client.call_api(
            "/v3/orderbook",
            "GET",
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type="RelayerApiOrderbookResponseSchema",
            auth_settings=auth_settings,
            async_req=local_var_params.get("async_req"),
            _return_http_data_only=local_var_params.get(
                "_return_http_data_only"
            ),
            _preload_content=local_var_params.get("_preload_content", True),
            _request_timeout=local_var_params.get("_request_timeout"),
            collection_formats=collection_formats,
        )

    def get_orders(self, **kwargs):
        """get_orders

        Retrieves a list of orders given query parameters. This endpoint
        should be paginated. For querying an entire
        orderbook snapshot, the orderbook endpoint
        is recommended. If both makerAssetData and takerAssetData are
        specified, returned orders will be sorted by price determined by
        (takerTokenAmount/makerTokenAmount) in ascending order. By default,
        orders returned by this endpoint are unsorted.
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass `async_req` = **True**

        >>> thread = api.get_orders(async_req=True) # doctest: +SKIP
        >>> result = thread.get() # doctest: +SKIP

        :param bool async_req: Whether request should be asynchronous.
        :param str maker_asset_proxy_id: The maker
            `asset proxy id
            <https://0x.org/docs/tools/0x.js#enumeration-assetproxyid>`__
            (example: "0xf47261b0" for ERC20, "0x02571792" for ERC721).
        :param str taker_asset_proxy_id: The taker asset
            `asset proxy id
            <https://0x.org/docs/tools/0x.js#enumeration-assetproxyid>`__
            (example: "0xf47261b0" for ERC20, "0x02571792" for ERC721).
        :param str maker_asset_address: The contract address for the maker asset.
        :param str taker_asset_address: The contract address for the taker asset.
        :param str exchange_address: Contract address for the exchange
            contract.
        :param str sender_address: Same as senderAddress in the
            `0x Protocol v3 Specification
            <https://github.com/0xProject/0x-protocol-specification/blob/
            master/v3/v3-specification.md#order-message-format>`__
        :param str maker_asset_data: Same as makerAssetData in the
            `0x Protocol v3 Specification
            <https://github.com/0xProject/0x-protocol-specification/blob/
            master/v3/v3-specification.md#order-message-format>`__
        :param str taker_asset_data: Same as takerAssetData in the
            `0x Protocol v3 Specification
            <https://github.com/0xProject/0x-protocol-specification/blob/
            master/v3/v3-specification.md#order-message-format>`__
        :param str trader_asset_data: Same as traderAssetData in the [0x
            `0x Protocol v3 Specification
            <https://github.com/0xProject/0x-protocol-specification/blob/
            master/v3/v3-specification.md#order-message-format>`__
        :param str maker_address: Same as makerAddress in the
            `0x Protocol v3 Specification
            <https://github.com/0xProject/0x-protocol-specification/blob/
            master/v3/v3-specification.md#order-message-format>`__
        :param str taker_address: Same as takerAddress in the
            `0x Protocol v3 Specification
            <https://github.com/0xProject/0x-protocol-specification/blob/
            master/v3/v3-specification.md#order-message-format>`__
        :param str trader_address: Same as traderAddress in the
            `0x Protocol v3 Specification
            <https://github.com/0xProject/0x-protocol-specification/blob/
            master/v3/v3-specification.md#order-message-format>`__
        :param str fee_recipient_address: Same as feeRecipientAddress in the
            `0x Protocol v3 Specification
            <https://github.com/0xProject/0x-protocol-specification/blob/
            master/v3/v3-specification.md#order-message-format>`__
        :param int network_id: The id of the Ethereum network
        :param int page: The number of the page to request in the collection.
        :param int per_page: The number of records to return per page.

        :return: :class:`RelayerApiOrdersResponseSchema`.
            If the method is called asynchronously, returns the request thread.
        """
        kwargs["_return_http_data_only"] = True
        if kwargs.get("async_req"):
            return self.get_orders_with_http_info(**kwargs)
        else:
            (data) = self.get_orders_with_http_info(**kwargs)
            return data

    def get_orders_with_http_info(self, **kwargs):
        """get_orders

        Retrieves a list of orders given query parameters. This endpoint
        should be paginated. For querying an entire
        orderbook snapshot, the orderbook endpoint
        is recommended. If both makerAssetData and takerAssetData are
        specified, returned orders will be sorted by price determined by
        (takerTokenAmount/makerTokenAmount) in ascending order. By default,
        orders returned by this endpoint are unsorted.
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass `async_req` = **True**

        >>> thread = api.get_orders_with_http_info(async_req=True) # doctest: +SKIP
        >>> result = thread.get() # doctest: +SKIP

        :param bool async_req: Whether request should be asynchronous.
        :param str maker_asset_proxy_id: The maker
            `asset proxy id
            <https://0x.org/docs/tools/0x.js#enumeration-assetproxyid>`__
            (example: "0xf47261b0" for ERC20, "0x02571792" for ERC721).
        :param str taker_asset_proxy_id: The taker asset
            `asset proxy id
            <https://0x.org/docs/tools/0x.js#enumeration-assetproxyid>`__
            (example: "0xf47261b0" for ERC20, "0x02571792" for ERC721).
        :param str maker_asset_address: The contract address for the maker asset.
        :param str taker_asset_address: The contract address for the taker asset.
        :param str exchange_address: Same as exchangeAddress in the [0x
            `0x Protocol v3 Specification
            <https://github.com/0xProject/0x-protocol-specification/blob/
            master/v3/v3-specification.md#order-message-format>`__
        :param str sender_address: Same as senderAddress in the
            `0x Protocol v3 Specification
            <https://github.com/0xProject/0x-protocol-specification/blob/
            master/v3/v3-specification.md#order-message-format>`__
        :param str maker_asset_data: Same as makerAssetData in the
            `0x Protocol v3 Specification
            <https://github.com/0xProject/0x-protocol-specification/blob/
            master/v3/v3-specification.md#order-message-format>`__
        :param str taker_asset_data: Same as takerAssetData in the
            `0x Protocol v3 Specification
            <https://github.com/0xProject/0x-protocol-specification/blob/
            master/v3/v3-specification.md#order-message-format>`__
        :param str trader_asset_data: Same as traderAssetData in the
            `0x Protocol v3 Specification
            <https://github.com/0xProject/0x-protocol-specification/blob/
            master/v3/v3-specification.md#order-message-format>`__
        :param str maker_address: Same as makerAddress in the
            `0x Protocol v3 Specification
            <https://github.com/0xProject/0x-protocol-specification/blob/
            master/v3/v3-specification.md#order-message-format>`__
        :param str taker_address: Same as takerAddress in the
            `0x Protocol v3 Specification
            <https://github.com/0xProject/0x-protocol-specification/blob/
            master/v3/v3-specification.md#order-message-format>`__
        :param str trader_address: Same as traderAddress in the
            `0x Protocol v3 Specification
            <https://github.com/0xProject/0x-protocol-specification/blob/
            master/v3/v3-specification.md#order-message-format>`__
        :param str fee_recipient_address: Same as feeRecipientAddress in the
            `0x Protocol v3 Specification
            <https://github.com/0xProject/0x-protocol-specification/blob/
            master/v3/v3-specification.md#order-message-format>`__
        :param int network_id: The id of the Ethereum network
        :param int page: The number of the page to request in the collection.
        :param int per_page: The number of records to return per page.

        :return: A tuple consisting of a
            :class:`RelayerApiOrdersResponseSchema`, an HTTP status code
            integer, and a collection of HTTP headers.  If the method is
            called asynchronously returns the request thread.
        """

        local_var_params = locals()

        all_params = [
            "maker_asset_proxy_id",
            "taker_asset_proxy_id",
            "maker_asset_address",
            "taker_asset_address",
            "exchange_address",
            "sender_address",
            "maker_asset_data",
            "taker_asset_data",
            "trader_asset_data",
            "maker_address",
            "taker_address",
            "trader_address",
            "fee_recipient_address",
            "network_id",
            "page",
            "per_page",
        ]
        all_params.append("async_req")
        all_params.append("_return_http_data_only")
        all_params.append("_preload_content")
        all_params.append("_request_timeout")

        for key, val in six.iteritems(local_var_params["kwargs"]):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method get_orders" % key
                )
            local_var_params[key] = val
        del local_var_params["kwargs"]

        collection_formats = {}

        path_params = {}

        query_params = []
        if "maker_asset_proxy_id" in local_var_params:
            query_params.append(
                ("makerAssetProxyId", local_var_params["maker_asset_proxy_id"])
            )
        if "taker_asset_proxy_id" in local_var_params:
            query_params.append(
                ("takerAssetProxyId", local_var_params["taker_asset_proxy_id"])
            )
        if "maker_asset_address" in local_var_params:
            query_params.append(
                ("makerAssetAddress", local_var_params["maker_asset_address"])
            )
        if "taker_asset_address" in local_var_params:
            query_params.append(
                ("takerAssetAddress", local_var_params["taker_asset_address"])
            )
        if "exchange_address" in local_var_params:
            query_params.append(
                ("exchangeAddress", local_var_params["exchange_address"])
            )
        if "sender_address" in local_var_params:
            query_params.append(
                ("senderAddress", local_var_params["sender_address"])
            )
        if "maker_asset_data" in local_var_params:
            query_params.append(
                ("makerAssetData", local_var_params["maker_asset_data"])
            )
        if "taker_asset_data" in local_var_params:
            query_params.append(
                ("takerAssetData", local_var_params["taker_asset_data"])
            )
        if "trader_asset_data" in local_var_params:
            query_params.append(
                ("traderAssetData", local_var_params["trader_asset_data"])
            )
        if "maker_address" in local_var_params:
            query_params.append(
                ("makerAddress", local_var_params["maker_address"])
            )
        if "taker_address" in local_var_params:
            query_params.append(
                ("takerAddress", local_var_params["taker_address"])
            )
        if "trader_address" in local_var_params:
            query_params.append(
                ("traderAddress", local_var_params["trader_address"])
            )
        if "fee_recipient_address" in local_var_params:
            query_params.append(
                (
                    "feeRecipientAddress",
                    local_var_params["fee_recipient_address"],
                )
            )
        if "network_id" in local_var_params:
            query_params.append(("networkId", local_var_params["network_id"]))
        if "page" in local_var_params:
            query_params.append(("page", local_var_params["page"]))
        if "per_page" in local_var_params:
            query_params.append(("perPage", local_var_params["per_page"]))

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params["Accept"] = self.api_client.select_header_accept(
            ["application/json"]
        )

        # Authentication setting
        auth_settings = []

        return self.api_client.call_api(
            "/v3/orders",
            "GET",
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type="RelayerApiOrdersResponseSchema",
            auth_settings=auth_settings,
            async_req=local_var_params.get("async_req"),
            _return_http_data_only=local_var_params.get(
                "_return_http_data_only"
            ),
            _preload_content=local_var_params.get("_preload_content", True),
            _request_timeout=local_var_params.get("_request_timeout"),
            collection_formats=collection_formats,
        )

    def post_order(self, **kwargs):
        """post_order

        Submit a signed order to the relayer.
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass `async_req` = **True**

        >>> thread = api.post_order(async_req=True) # doctest: +SKIP
        >>> result = thread.get() # doctest: +SKIP

        :param bool async_req: Whether request should be asynchronous.
        :param int network_id: The id of the Ethereum network
        :param signed_order_schema: Instance of :class:`SignedOrderSchema`.
            A valid signed 0x order based on the schema.

        :return: None.
            If the method is called asynchronously, returns the request thread.
        """
        kwargs["_return_http_data_only"] = True
        if kwargs.get("async_req"):
            return self.post_order_with_http_info(**kwargs)
        else:
            (data) = self.post_order_with_http_info(**kwargs)
            return data

    def post_order_with_http_info(self, **kwargs):
        """post_order

        Submit a signed order to the relayer.
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass `async_req` = **True**

        >>> thread = api.post_order_with_http_info(async_req=True) # doctest: +SKIP
        >>> result = thread.get() # doctest: +SKIP

        :param bool async_req: Whether request should be asynchronous.
        :param int network_id: The id of the Ethereum network
        :param signed_order_schema: Instance of :class:`SignedOrderSchema`
            A valid signed 0x order based on the schema.

        :return: A tuple consisting of the response data (always empty for this
            method), an HTTP status code integer, and a collection of HTTP
            headers.  If the method is called asynchronously returns the
            request thread.
        """

        local_var_params = locals()

        all_params = ["network_id", "signed_order_schema"]
        all_params.append("async_req")
        all_params.append("_return_http_data_only")
        all_params.append("_preload_content")
        all_params.append("_request_timeout")

        for key, val in six.iteritems(local_var_params["kwargs"]):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method post_order" % key
                )
            local_var_params[key] = val
        del local_var_params["kwargs"]

        collection_formats = {}

        path_params = {}

        query_params = []
        if "network_id" in local_var_params:
            query_params.append(("networkId", local_var_params["network_id"]))

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        if "signed_order_schema" in local_var_params:
            body_params = local_var_params["signed_order_schema"]
        # HTTP header `Accept`
        header_params["Accept"] = self.api_client.select_header_accept(
            ["application/json"]
        )

        # HTTP header `Content-Type`
        header_params[
            "Content-Type"
        ] = self.api_client.select_header_content_type(["application/json"])

        # Authentication setting
        auth_settings = []

        return self.api_client.call_api(
            "/v3/order",
            "POST",
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type=None,
            auth_settings=auth_settings,
            async_req=local_var_params.get("async_req"),
            _return_http_data_only=local_var_params.get(
                "_return_http_data_only"
            ),
            _preload_content=local_var_params.get("_preload_content", True),
            _request_timeout=local_var_params.get("_request_timeout"),
            collection_formats=collection_formats,
        )



# coding: utf-8


import pprint
import re  # noqa: F401

import six


class RelayerApiErrorResponseSchemaValidationErrors(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {"field": "str", "code": "int", "reason": "str"}

    attribute_map = {"field": "field", "code": "code", "reason": "reason"}

    def __init__(self, field=None, code=None, reason=None):  # noqa: E501
        """RelayerApiErrorResponseSchemaValidationErrors - a model defined in OpenAPI"""  # noqa: E501

        self._field = None
        self._code = None
        self._reason = None
        self.discriminator = None

        self.field = field
        self.code = code
        self.reason = reason

    @property
    def field(self):
        """Gets the field of this RelayerApiErrorResponseSchemaValidationErrors.  # noqa: E501


        :return: The field of this RelayerApiErrorResponseSchemaValidationErrors.  # noqa: E501
        :rtype: str
        """
        return self._field

    @field.setter
    def field(self, field):
        """Sets the field of this RelayerApiErrorResponseSchemaValidationErrors.


        :param field: The field of this RelayerApiErrorResponseSchemaValidationErrors.  # noqa: E501
        :type: str
        """
        if field is None:
            raise ValueError(
                "Invalid value for `field`, must not be `None`"
            )  # noqa: E501

        self._field = field

    @property
    def code(self):
        """Gets the code of this RelayerApiErrorResponseSchemaValidationErrors.  # noqa: E501


        :return: The code of this RelayerApiErrorResponseSchemaValidationErrors.  # noqa: E501
        :rtype: int
        """
        return self._code

    @code.setter
    def code(self, code):
        """Sets the code of this RelayerApiErrorResponseSchemaValidationErrors.


        :param code: The code of this RelayerApiErrorResponseSchemaValidationErrors.  # noqa: E501
        :type: int
        """
        if code is None:
            raise ValueError(
                "Invalid value for `code`, must not be `None`"
            )  # noqa: E501
        if code is not None and code > 1006:  # noqa: E501
            raise ValueError(
                "Invalid value for `code`, must be a value less than or equal to `1006`"
            )  # noqa: E501
        if code is not None and code < 1000:  # noqa: E501
            raise ValueError(
                "Invalid value for `code`, must be a value greater than or equal to `1000`"
            )  # noqa: E501

        self._code = code

    @property
    def reason(self):
        """Gets the reason of this RelayerApiErrorResponseSchemaValidationErrors.  # noqa: E501


        :return: The reason of this RelayerApiErrorResponseSchemaValidationErrors.  # noqa: E501
        :rtype: str
        """
        return self._reason

    @reason.setter
    def reason(self, reason):
        """Sets the reason of this RelayerApiErrorResponseSchemaValidationErrors.


        :param reason: The reason of this RelayerApiErrorResponseSchemaValidationErrors.  # noqa: E501
        :type: str
        """
        if reason is None:
            raise ValueError(
                "Invalid value for `reason`, must not be `None`"
            )  # noqa: E501

        self._reason = reason

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(
                    map(
                        lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                        value,
                    )
                )
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(
                    map(
                        lambda item: (item[0], item[1].to_dict())
                        if hasattr(item[1], "to_dict")
                        else item,
                        value.items(),
                    )
                )
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(
            other, RelayerApiErrorResponseSchemaValidationErrors
        ):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

# flake8: noqa

r"""A Python client for interacting with SRA-compatible Relayers.

0x Protocol is an open standard.  Many Relayers opt to implementing a set of
`Standard Relayer API (SRA)
<http://sra-spec.s3-website-us-east-1.amazonaws.com/>`_ endpoints, to make it
easier for anyone to source liquidity that conforms to the 0x order format.
Here, we will show you how you can use the `0x-sra-client`:code: module to
interact with 0x relayers that implement the SRA specification.

Setup
-----

Install the package with pip::

    pip install 0x-sra-client

To interact with a 0x Relayer, you need the HTTP endpoint of the Relayer you'd
like to connect to (eg https://api.radarrelay.com/0x/v3).

For testing one can use the `0x-launch-kit-backend
<https://github.com/0xProject/0x-launch-kit-backend#table-of-contents/>`_ to host
orders locally.  The examples below assume that this server is running locally
and listening on port 3000, so the Relayer URL they use is
`http://localhost:3000`:code:.

By default, Launch Kit will connect to Kovan via Infura.  However, it can be
configured to connect to any JSON-RPC endpoint, on any network.  The examples
below assume that Launch Kit is connected to a Ganache development network
accessible at `http://localhost:8545`:code:.

To replicate this setup, one could run the following commands:

::

    docker run -d -p 8545:8545 0xorg/ganache-cli

    docker run -d --network host \
        -e RPC_URL=http://localhost:8545 \
        -e NETWORK_ID=50 \
        -e WHITELIST_ALL_TOKENS=True \
        0xorg/launch-kit-ci

(Note: This will only work on Linux, because `--network host`:code: only works
on Linux.  For other platforms one would have to clone `the 0x-launch-kit-backend
repository <https://github.com/0xProject/0x-launch-kit-backend>`_ and build and start
the server.)

Configure and create an API client instance
-------------------------------------------

>>> from zero_ex.sra_client import ApiClient, Configuration, DefaultApi
>>> config = Configuration()
>>> config.host = "http://localhost:3000"
>>> relayer = DefaultApi(ApiClient(config))

Preparing to trade
------------------

Making and taking orders induces the SRA endpoint to deal with the Ethereum
network.  Before we can start trading, we need to do a few things with the
network directly.

To start, connect to the Ethereum network:

>>> from web3 import HTTPProvider, Web3
>>> eth_node = HTTPProvider("http://localhost:8545")

What network is it?

>>> from zero_ex.contract_addresses import NetworkId
>>> network_id = NetworkId.GANACHE  # you might use .MAINNET or .KOVAN

For our Maker role, we'll just use the first address available in the node:

>>> maker_address = Web3(eth_node).eth.accounts[0]

The 0x Ganache snapshot loaded into our eth_node has a pre-loaded ZRX balance
for this account, so the example orders below have the maker trading away ZRX.
Before such an order can be valid, though, the maker must give the 0x contracts
permission to trade their ZRX tokens:

>>> from zero_ex.contract_addresses import network_to_addresses
>>> contract_addresses = network_to_addresses(network_id)
>>>
>>> from zero_ex.contract_artifacts import abi_by_name
>>> zrx_token_contract = Web3(eth_node).eth.contract(
...    address=Web3.toChecksumAddress(contract_addresses.zrx_token),
...    abi=abi_by_name("ZRXToken")
... )
>>>
>>> zrx_token_contract.functions.approve(
...     Web3.toChecksumAddress(contract_addresses.erc20_proxy),
...     1000000000000000000
... ).transact(
...     {"from": Web3.toChecksumAddress(maker_address)}
... )
HexBytes('0x...')

Post Order
-----------

Post an order for our Maker to trade ZRX for WETH:

>>> from zero_ex.contract_wrappers.exchange.types import Order
>>> from zero_ex.contract_wrappers.order_conversions import order_to_jsdict
>>> from zero_ex.order_utils import (
...     asset_data_utils,
...     sign_hash)
>>> import random
>>> from datetime import datetime, timedelta
>>> order = Order(
...     makerAddress=maker_address,
...     takerAddress="0x0000000000000000000000000000000000000000",
...     senderAddress="0x0000000000000000000000000000000000000000",
...     exchangeAddress=contract_addresses.exchange,
...     feeRecipientAddress="0x0000000000000000000000000000000000000000",
...     makerAssetData=asset_data_utils.encode_erc20(
...         contract_addresses.zrx_token
...     ),
...     makerFeeAssetData=asset_data_utils.encode_erc20('0x'+'00'*20),
...     takerAssetData=asset_data_utils.encode_erc20(
...         contract_addresses.ether_token
...     ),
...     takerFeeAssetData=asset_data_utils.encode_erc20('0x'+'00'*20),
...     salt=random.randint(1, 100000000000000000),
...     makerFee=0,
...     takerFee=0,
...     makerAssetAmount=2,
...     takerAssetAmount=2,
...     expirationTimeSeconds=round(
...         (datetime.utcnow() + timedelta(days=1)).timestamp()
...     )
... )

>>> from zero_ex.order_utils import generate_order_hash_hex
>>> order_hash_hex = generate_order_hash_hex(
...     order, contract_addresses.exchange, Web3(eth_node).eth.chainId
... )
>>> relayer.post_order_with_http_info(
...     network_id=network_id.value,
...     signed_order_schema=order_to_jsdict(
...         order=order,
...         exchange_address=contract_addresses.exchange,
...         signature=sign_hash(
...             eth_node, Web3.toChecksumAddress(maker_address), order_hash_hex
...         ),
...         chain_id=Web3(eth_node).eth.chainId,
...     )
... )[1]
200

Get Order
---------

(But first sleep for a moment, to give the test relayer a chance to start up.

>>> from time import sleep
>>> sleep(0.2)

This is necessary for automated verification of these examples.)

Retrieve the order we just posted:

>>> relayer.get_order("0x" + order_hash_hex)
{'meta_data': {'orderHash': '0x...',
               'remainingFillableTakerAssetAmount': '2'},
 'order': {'chainId': 50,
           'exchangeAddress': '0x...',
           'expirationTimeSeconds': '...',
           'feeRecipientAddress': '0x0000000000000000000000000000000000000000',
           'makerAddress': '0x...',
           'makerAssetAmount': '2',
           'makerAssetData': '0xf47261b0000000000000000000000000...',
           'makerFee': '0',
           'makerFeeAssetData': '0xf47261b0000000000000000000000000...',
           'salt': '...',
           'senderAddress': '0x0000000000000000000000000000000000000000',
           'signature': '0x...',
           'takerAddress': '0x0000000000000000000000000000000000000000',
           'takerAssetAmount': '2',
           'takerAssetData': '0xf47261b0000000000000000000000000...',
           'takerFee': '0',
           'takerFeeAssetData': '0xf47261b0000000000000000000000000...'}}

Get Orders
-----------

Retrieve all of the Relayer's orders, a set which at this point consists solely
of the one we just posted:

>>> relayer.get_orders()
{'records': [{'meta_data': {'orderHash': '0x...',
                            'remainingFillableTakerAssetAmount': '2'},
              'order': {'chainId': 50,
                        'exchangeAddress': '0x...',
                        'expirationTimeSeconds': '...',
                        'feeRecipientAddress': '0x0000000000000000000000000000000000000000',
                        'makerAddress': '0x...',
                        'makerAssetAmount': '2',
                        'makerAssetData': '0xf47261b000000000000000000000000...',
                        'makerFee': '0',
                        'makerFeeAssetData': '0xf47261b000000000000000000000000...',
                        'salt': '...',
                        'senderAddress': '0x0000000000000000000000000000000000000000',
                        'signature': '0x...',
                        'takerAddress': '0x0000000000000000000000000000000000000000',
                        'takerAssetAmount': '2',
                        'takerAssetData': '0xf47261b0000000000000000000000000...',
                        'takerFee': '0',
                        'takerFeeAssetData': '0xf47261b0000000000000000000000000...'}}...]}

Get Asset Pairs
---------------

Get all of the Relayer's available asset pairs, which here means just WETH and
ZRX, since that's all there is on this Relayer's order book:

>>> relayer.get_asset_pairs()
{'records': [{'assetDataA': {'assetData': '0xf47261b0000000000000000000000000...',
                             'maxAmount': '115792089237316195423570985008687907853269984665640564039457584007913129639936',
                             'minAmount': '0',
                             'precision': 18},
              'assetDataB': {'assetData': '0xf47261b0000000000000000000000000...',
                             'maxAmount': '115792089237316195423570985008687907853269984665640564039457584007913129639936',
                             'minAmount': '0',
                             'precision': 18}}]}
>>> asset_data_utils.decode_erc20_asset_data(
...     relayer.get_asset_pairs().records[0]['assetDataA']['assetData']
... ).token_address == contract_addresses.zrx_token
True
>>> asset_data_utils.decode_erc20_asset_data(
...     relayer.get_asset_pairs().records[0]['assetDataB']['assetData']
... ).token_address == contract_addresses.ether_token
True

Get Orderbook
-------------

Get the Relayer's order book for the WETH/ZRX asset pair (which, again,
consists just of our order):

>>> orderbook = relayer.get_orderbook(
...     base_asset_data= "0x" + asset_data_utils.encode_erc20(
...         contract_addresses.ether_token
...     ).hex(),
...     quote_asset_data= "0x" + asset_data_utils.encode_erc20(
...         contract_addresses.zrx_token
...     ).hex(),
... )
>>> orderbook
{'asks': {'records': [...]},
 'bids': {'records': [{'meta_data': {'orderHash': '0x...',
                                     'remainingFillableTakerAssetAmount': '2'},
                       'order': {'chainId': 50,
                                 'exchangeAddress': '0x...',
                                 'expirationTimeSeconds': '...',
                                 'feeRecipientAddress': '0x0000000000000000000000000000000000000000',
                                 'makerAddress': '0x...',
                                 'makerAssetAmount': '2',
                                 'makerAssetData': '0xf47261b0000000000000000000000000...',
                                 'makerFee': '0',
                                 'makerFeeAssetData': '0xf47261b0000000000000000000000000...',
                                 'salt': '...',
                                 'senderAddress': '0x0000000000000000000000000000000000000000',
                                 'signature': '0x...',
                                 'takerAddress': '0x0000000000000000000000000000000000000000',
                                 'takerAssetAmount': '2',
                                 'takerAssetData': '0xf47261b0000000000000000000000000...',
                                 'takerFee': '0',
                                 'takerFeeAssetData': '0xf47261b0000000000000000000000000...'}}...]}}

Select an order from the orderbook
^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^

>>> from zero_ex.contract_wrappers.order_conversions import jsdict_to_order
>>> order = jsdict_to_order(orderbook.bids.records[0].order)
>>> from pprint import pprint
>>> pprint(order)
{'chainId': 50,
 'expirationTimeSeconds': ...,
 'feeRecipientAddress': '0x0000000000000000000000000000000000000000',
 'makerAddress': '0x...',
 'makerAssetAmount': 2,
 'makerAssetData': b...
 'makerFee': 0,
 'makerFeeAssetData': b...
 'salt': ...,
 'senderAddress': '0x0000000000000000000000000000000000000000',
 'signature': '0x...',
 'takerAddress': '0x0000000000000000000000000000000000000000',
 'takerAssetAmount': 2,
 'takerAssetData': b...,
 'takerFee': 0,
 'takerFeeAssetData': b...}

Filling or Cancelling an Order
------------------------------

Fills and cancels are triggered by dealing directly with the 0x Exchange
contract, not by going through a Relayer.

See `the 0x-contract-wrappers documentation
<http://0x-contract-wrappers-py.s3-website-us-east-1.amazonaws.com/>`_ for more
examples.

Filling
^^^^^^^

>>> taker_address = Web3(eth_node).eth.accounts[1]

Our taker will take a ZRX/WETH order, but it doesn't have any WETH yet.  By
depositing some ether into the WETH contract, it will be given some WETH to
trade with:

>>> weth_instance = Web3(eth_node).eth.contract(
...    address=Web3.toChecksumAddress(contract_addresses.ether_token),
...    abi=abi_by_name("WETH9")
... )
>>> weth_instance.functions.deposit().transact(
...     {"from": Web3.toChecksumAddress(taker_address),
...      "value": 1000000000000000000}
... )
HexBytes('0x...')

Next the taker needs to give the 0x contracts permission to trade their WETH:

>>> weth_instance.functions.approve(
...     Web3.toChecksumAddress(contract_addresses.erc20_proxy),
...     1000000000000000000
... ).transact(
...     {"from": Web3.toChecksumAddress(taker_address)}
... )
HexBytes('0x...')

Now the taker is ready to trade.

Recall that in a previous example we selected a specific order from the order
book.  Now let's have the taker fill it:

>>> from zero_ex.contract_wrappers import TxParams
>>> from zero_ex.contract_wrappers.exchange import Exchange
>>> from zero_ex.order_utils import Order
>>> exchange = Exchange(
...     web3_or_provider=eth_node,
...     contract_address=network_to_addresses(NetworkId.GANACHE).exchange
... )

(Due to `an Issue with the Launch Kit Backend
<https://github.com/0xProject/0x-launch-kit-backend/issues/73>`_, we need to
checksum the address in the order before filling it.)
>>> order['makerAddress'] = Web3.toChecksumAddress(order['makerAddress'])

>>> exchange.fill_order.send_transaction(
...     order=order,
...     taker_asset_fill_amount=order['makerAssetAmount']/2, # note the half fill
...     signature=bytes.fromhex(order['signature'].replace('0x', '')),
...     tx_params=TxParams(from_=taker_address)
... )
HexBytes('0x...')

Cancelling
^^^^^^^^^^

Note that the above fill was partial: it only filled half of the order.  Now
we'll have our maker cancel the remaining order:

>>> exchange.cancel_order.send_transaction(
...     order=order,
...     tx_params=TxParams(from_=maker_address)
... )
HexBytes('0x...')

"""  # noqa: E501 (line too long)

from __future__ import absolute_import

__version__ = "1.0.0"

# import apis into sdk package
from .api.default_api import DefaultApi

# import ApiClient
from .api_client import ApiClient
from .configuration import Configuration

# import models into sdk package
from .models.order_schema import OrderSchema
from .models.paginated_collection_schema import PaginatedCollectionSchema
from .models.relayer_api_asset_data_pairs_response_schema import (
    RelayerApiAssetDataPairsResponseSchema,
)
from .models.relayer_api_asset_data_trade_info_schema import (
    RelayerApiAssetDataTradeInfoSchema,
)
from .models.relayer_api_error_response_schema import (
    RelayerApiErrorResponseSchema,
)
from .models.relayer_api_error_response_schema_validation_errors import (
    RelayerApiErrorResponseSchemaValidationErrors,
)
from .models.relayer_api_fee_recipients_response_schema import (
    RelayerApiFeeRecipientsResponseSchema,
)
from .models.relayer_api_order_config_payload_schema import (
    RelayerApiOrderConfigPayloadSchema,
)
from .models.relayer_api_order_config_response_schema import (
    RelayerApiOrderConfigResponseSchema,
)
from .models.relayer_api_order_schema import RelayerApiOrderSchema
from .models.relayer_api_orderbook_response_schema import (
    RelayerApiOrderbookResponseSchema,
)
from .models.relayer_api_orders_channel_subscribe_payload_schema import (
    RelayerApiOrdersChannelSubscribePayloadSchema,
)
from .models.relayer_api_orders_channel_subscribe_schema import (
    RelayerApiOrdersChannelSubscribeSchema,
)
from .models.relayer_api_orders_channel_update_schema import (
    RelayerApiOrdersChannelUpdateSchema,
)
from .models.relayer_api_orders_response_schema import (
    RelayerApiOrdersResponseSchema,
)
from .models.signed_order_schema import SignedOrderSchema



# coding: utf-8


import pprint
import re  # noqa: F401

import six


class RelayerApiAssetDataTradeInfoSchema(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        "asset_data": "str",
        "min_amount": "str",
        "max_amount": "str",
        "precision": "float",
    }

    attribute_map = {
        "asset_data": "assetData",
        "min_amount": "minAmount",
        "max_amount": "maxAmount",
        "precision": "precision",
    }

    def __init__(
        self, asset_data=None, min_amount=None, max_amount=None, precision=None
    ):  # noqa: E501
        """RelayerApiAssetDataTradeInfoSchema - a model defined in OpenAPI"""  # noqa: E501

        self._asset_data = None
        self._min_amount = None
        self._max_amount = None
        self._precision = None
        self.discriminator = None

        self.asset_data = asset_data
        if min_amount is not None:
            self.min_amount = min_amount
        if max_amount is not None:
            self.max_amount = max_amount
        if precision is not None:
            self.precision = precision

    @property
    def asset_data(self):
        """Gets the asset_data of this RelayerApiAssetDataTradeInfoSchema.  # noqa: E501


        :return: The asset_data of this RelayerApiAssetDataTradeInfoSchema.  # noqa: E501
        :rtype: str
        """
        return self._asset_data

    @asset_data.setter
    def asset_data(self, asset_data):
        """Sets the asset_data of this RelayerApiAssetDataTradeInfoSchema.


        :param asset_data: The asset_data of this RelayerApiAssetDataTradeInfoSchema.  # noqa: E501
        :type: str
        """
        if asset_data is None:
            raise ValueError(
                "Invalid value for `asset_data`, must not be `None`"
            )  # noqa: E501
        if asset_data is not None and not re.search(
            r"^0x(([0-9a-f][0-9a-f])+)?$", asset_data
        ):  # noqa: E501
            raise ValueError(
                r"Invalid value for `asset_data`, must be a follow pattern or equal to `/^0x(([0-9a-f][0-9a-f])+)?$/`"
            )  # noqa: E501

        self._asset_data = asset_data

    @property
    def min_amount(self):
        """Gets the min_amount of this RelayerApiAssetDataTradeInfoSchema.  # noqa: E501


        :return: The min_amount of this RelayerApiAssetDataTradeInfoSchema.  # noqa: E501
        :rtype: str
        """
        return self._min_amount

    @min_amount.setter
    def min_amount(self, min_amount):
        """Sets the min_amount of this RelayerApiAssetDataTradeInfoSchema.


        :param min_amount: The min_amount of this RelayerApiAssetDataTradeInfoSchema.  # noqa: E501
        :type: str
        """
        if min_amount is not None and not re.search(
            r"^\\d+$", min_amount
        ):  # noqa: E501
            raise ValueError(
                r"Invalid value for `min_amount`, must be a follow pattern or equal to `/^\\d+$/`"
            )  # noqa: E501

        self._min_amount = min_amount

    @property
    def max_amount(self):
        """Gets the max_amount of this RelayerApiAssetDataTradeInfoSchema.  # noqa: E501


        :return: The max_amount of this RelayerApiAssetDataTradeInfoSchema.  # noqa: E501
        :rtype: str
        """
        return self._max_amount

    @max_amount.setter
    def max_amount(self, max_amount):
        """Sets the max_amount of this RelayerApiAssetDataTradeInfoSchema.


        :param max_amount: The max_amount of this RelayerApiAssetDataTradeInfoSchema.  # noqa: E501
        :type: str
        """
        if max_amount is not None and not re.search(
            r"^\\d+$", max_amount
        ):  # noqa: E501
            raise ValueError(
                r"Invalid value for `max_amount`, must be a follow pattern or equal to `/^\\d+$/`"
            )  # noqa: E501

        self._max_amount = max_amount

    @property
    def precision(self):
        """Gets the precision of this RelayerApiAssetDataTradeInfoSchema.  # noqa: E501


        :return: The precision of this RelayerApiAssetDataTradeInfoSchema.  # noqa: E501
        :rtype: float
        """
        return self._precision

    @precision.setter
    def precision(self, precision):
        """Sets the precision of this RelayerApiAssetDataTradeInfoSchema.


        :param precision: The precision of this RelayerApiAssetDataTradeInfoSchema.  # noqa: E501
        :type: float
        """

        self._precision = precision

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(
                    map(
                        lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                        value,
                    )
                )
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(
                    map(
                        lambda item: (item[0], item[1].to_dict())
                        if hasattr(item[1], "to_dict")
                        else item,
                        value.items(),
                    )
                )
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, RelayerApiAssetDataTradeInfoSchema):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8


import pprint
import re  # noqa: F401

import six


class RelayerApiOrderbookResponseSchema(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        "bids": "RelayerApiOrdersResponseSchema",
        "asks": "RelayerApiOrdersResponseSchema",
    }

    attribute_map = {"bids": "bids", "asks": "asks"}

    def __init__(self, bids=None, asks=None):  # noqa: E501
        """RelayerApiOrderbookResponseSchema - a model defined in OpenAPI"""  # noqa: E501

        self._bids = None
        self._asks = None
        self.discriminator = None

        self.bids = bids
        self.asks = asks

    @property
    def bids(self):
        """Gets the bids of this RelayerApiOrderbookResponseSchema.  # noqa: E501


        :return: The bids of this RelayerApiOrderbookResponseSchema.  # noqa: E501
        :rtype: RelayerApiOrdersResponseSchema
        """
        return self._bids

    @bids.setter
    def bids(self, bids):
        """Sets the bids of this RelayerApiOrderbookResponseSchema.


        :param bids: The bids of this RelayerApiOrderbookResponseSchema.  # noqa: E501
        :type: RelayerApiOrdersResponseSchema
        """
        if bids is None:
            raise ValueError(
                "Invalid value for `bids`, must not be `None`"
            )  # noqa: E501

        self._bids = bids

    @property
    def asks(self):
        """Gets the asks of this RelayerApiOrderbookResponseSchema.  # noqa: E501


        :return: The asks of this RelayerApiOrderbookResponseSchema.  # noqa: E501
        :rtype: RelayerApiOrdersResponseSchema
        """
        return self._asks

    @asks.setter
    def asks(self, asks):
        """Sets the asks of this RelayerApiOrderbookResponseSchema.


        :param asks: The asks of this RelayerApiOrderbookResponseSchema.  # noqa: E501
        :type: RelayerApiOrdersResponseSchema
        """
        if asks is None:
            raise ValueError(
                "Invalid value for `asks`, must not be `None`"
            )  # noqa: E501

        self._asks = asks

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(
                    map(
                        lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                        value,
                    )
                )
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(
                    map(
                        lambda item: (item[0], item[1].to_dict())
                        if hasattr(item[1], "to_dict")
                        else item,
                        value.items(),
                    )
                )
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, RelayerApiOrderbookResponseSchema):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



#!/usr/bin/env python
# coding: utf-8

"""setuptools module for sra_client package."""

# pylint: disable=import-outside-toplevel
# we import things outside of top-level because 3rd party libs may not yet be
# installed when you invoke this script

import subprocess  # nosec
import distutils.command.build_py
from distutils.command.clean import clean
from shutil import rmtree
from sys import exit  # pylint: disable=redefined-builtin
from urllib.request import urlopen
from urllib.error import URLError

from setuptools import setup, find_packages  # noqa: H301
from setuptools.command.test import test as TestCommand

NAME = "0x-sra-client"
VERSION = "4.0.0.dev0"
# To install the library, run the following
#
# python setup.py install
#
# prerequisite: setuptools
# http://pypi.python.org/pypi/setuptools

with open("README.md", "r") as file_handle:
    README_MD = file_handle.read()

REQUIRES = ["urllib3 >= 1.15", "six >= 1.10", "certifi", "python-dateutil"]


class CleanCommandExtension(clean):
    """Custom command to do custom cleanup."""

    def run(self):
        """Run the regular clean, followed by our custom commands."""
        super().run()
        rmtree("__pycache__", ignore_errors=True)
        rmtree(".mypy_cache", ignore_errors=True)
        rmtree(".tox", ignore_errors=True)
        rmtree(".pytest_cache", ignore_errors=True)
        rmtree("0x_sra_client.egg-info", ignore_errors=True)
        rmtree("build", ignore_errors=True)
        rmtree("dist", ignore_errors=True)
        subprocess.check_call(  # nosec
            ("docker-compose -f test/relayer/docker-compose.yml down").split()
        )
        subprocess.check_call(  # nosec
            ("docker-compose -f test/relayer/docker-compose.yml rm").split()
        )


class TestCommandExtension(TestCommand):
    """Run pytest tests."""

    def run_tests(self):
        """Invoke pytest."""
        import pytest

        exit(pytest.main(["--doctest-modules", "-rapP"]))
        #        show short test summary at end ^
        # above call commented out due to a problem with launch kit,
        # documented at
        # https://github.com/0xProject/0x-launch-kit-backend/issues/73


class TestPublishCommand(distutils.command.build_py.build_py):
    """Custom command to publish to test.pypi.org."""

    description = (
        "Publish dist/* to test.pypi.org. Run sdist & bdist_wheel first."
    )

    def run(self):
        """Run twine to upload to test.pypi.org."""
        subprocess.check_call(  # nosec
            (
                "twine upload --repository-url https://test.pypi.org/legacy/"
                + " --verbose dist/*"
            ).split()
        )


class StartTestRelayerCommand(distutils.command.build_py.build_py):
    """Custom command to boot up a local 0x-launch-kit-backend in docker."""

    description = "Run launch-kit daemon to support tests."

    def run(self):
        """Run `docker-compose up`."""
        subprocess.call(  # nosec
            ("docker-compose -f test/relayer/docker-compose.yml up -d").split()
        )
        launch_kit_ready = False
        print(
            "Waiting for Launch Kit Backend to start accepting connections...",
            flush=True,
        )
        while not launch_kit_ready:
            try:
                launch_kit_ready = (
                    urlopen(  # nosec
                        "http://localhost:3000/v3/asset_pairs"
                    ).getcode()
                    == 200
                )
            except URLError:
                continue
        print("done")


class StopTestRelayerCommand(distutils.command.build_py.build_py):
    """Custom command to tear down the 0x-launch-kit-backend test relayer."""

    description = "Tear down launch-kit daemon."

    def run(self):
        """Run `docker-compose down`."""
        subprocess.call(  # nosec
            ("docker-compose -f test/relayer/docker-compose.yml down").split()
        )


class LintCommand(distutils.command.build_py.build_py):
    """Custom setuptools command class for running linters."""

    description = "Run linters"

    def run(self):
        """Run linter shell commands."""
        lint_targets = "test src/zero_ex/sra_client/__init__.py setup.py"
        lint_commands = [
            # formatter:
            (
                f"black --line-length 79 --check --diff test {lint_targets}"
            ).split(),
            # style guide checker (formerly pep8):
            f"pycodestyle {lint_targets}".split(),
            # docstring style checker:
            f"pydocstyle {lint_targets}".split(),
            # static type checker:
            f"bandit -r {lint_targets}".split(),
            # general linter:
            f"pylint {lint_targets}".split(),
            # pylint takes relatively long to run, so it runs last, to enable
            # fast failures.
        ]

        for lint_command in lint_commands:
            print(
                "Running lint command `", " ".join(lint_command).strip(), "`"
            )
            subprocess.check_call(lint_command)  # nosec


class PublishCommand(distutils.command.build_py.build_py):
    """Custom command to publish to pypi.org."""

    description = "Publish dist/* to pypi.org. Run sdist & bdist_wheel first."

    def run(self):
        """Run twine to upload to pypi.org."""
        subprocess.check_call("twine upload dist/*".split())  # nosec


class PublishDocsCommand(distutils.command.build_py.build_py):
    """Custom command to publish docs to S3."""

    description = (
        "Publish docs to "
        + "http://0x-sra-client-py.s3-website-us-east-1.amazonaws.com/"
    )

    def run(self):
        """Run npm package `discharge` to build & upload docs."""
        subprocess.check_call("discharge deploy".split())  # nosec


setup(
    name=NAME,
    version=VERSION,
    description="Standard Relayer REST API Client",
    author="F. Eugene Aumson",
    author_email="feuGeneA@users.noreply.github.com",
    url=(
        "https://github.com/0xproject/0x-monorepo/tree/development"
        "/python-packages/sra_client"
    ),
    keywords=["OpenAPI", "OpenAPI-Generator", "Standard Relayer REST API"],
    install_requires=REQUIRES,
    namespace_packages=["zero_ex"],
    packages=find_packages("src"),
    package_dir={"": "src"},
    include_package_data=True,
    long_description=README_MD,
    long_description_content_type="text/markdown",
    cmdclass={
        "clean": CleanCommandExtension,
        "test_publish": TestPublishCommand,
        "publish": PublishCommand,
        "start_test_relayer": StartTestRelayerCommand,
        "stop_test_relayer": StopTestRelayerCommand,
        "lint": LintCommand,
        "publish_docs": PublishDocsCommand,
        "test": TestCommandExtension,
    },
    extras_require={
        "dev": [
            "0x-contract-artifacts",
            "0x-contract-addresses",
            "0x-order-utils",
            "web3",
            "bandit",
            "black",
            "coverage",
            "coveralls",
            "pycodestyle",
            "pydocstyle",
            "pylint",
            "pytest",
            "sphinx",
            "sphinx-autodoc-typehints",
        ]
    },
    command_options={
        "build_sphinx": {
            "source_dir": ("setup.py", "src"),
            "build_dir": ("setup.py", "build/docs"),
            "warning_is_error": ("setup.py", "true"),
        }
    },
)



# coding: utf-8


import pprint
import re  # noqa: F401

import six


class RelayerApiErrorResponseSchema(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        "code": "int",
        "reason": "str",
        "validation_errors": "list[RelayerApiErrorResponseSchemaValidationErrors]",
    }

    attribute_map = {
        "code": "code",
        "reason": "reason",
        "validation_errors": "validationErrors",
    }

    def __init__(
        self, code=None, reason=None, validation_errors=None
    ):  # noqa: E501
        """RelayerApiErrorResponseSchema - a model defined in OpenAPI"""  # noqa: E501

        self._code = None
        self._reason = None
        self._validation_errors = None
        self.discriminator = None

        self.code = code
        self.reason = reason
        if validation_errors is not None:
            self.validation_errors = validation_errors

    @property
    def code(self):
        """Gets the code of this RelayerApiErrorResponseSchema.  # noqa: E501


        :return: The code of this RelayerApiErrorResponseSchema.  # noqa: E501
        :rtype: int
        """
        return self._code

    @code.setter
    def code(self, code):
        """Sets the code of this RelayerApiErrorResponseSchema.


        :param code: The code of this RelayerApiErrorResponseSchema.  # noqa: E501
        :type: int
        """
        if code is None:
            raise ValueError(
                "Invalid value for `code`, must not be `None`"
            )  # noqa: E501
        if code is not None and code > 103:  # noqa: E501
            raise ValueError(
                "Invalid value for `code`, must be a value less than or equal to `103`"
            )  # noqa: E501
        if code is not None and code < 100:  # noqa: E501
            raise ValueError(
                "Invalid value for `code`, must be a value greater than or equal to `100`"
            )  # noqa: E501

        self._code = code

    @property
    def reason(self):
        """Gets the reason of this RelayerApiErrorResponseSchema.  # noqa: E501


        :return: The reason of this RelayerApiErrorResponseSchema.  # noqa: E501
        :rtype: str
        """
        return self._reason

    @reason.setter
    def reason(self, reason):
        """Sets the reason of this RelayerApiErrorResponseSchema.


        :param reason: The reason of this RelayerApiErrorResponseSchema.  # noqa: E501
        :type: str
        """
        if reason is None:
            raise ValueError(
                "Invalid value for `reason`, must not be `None`"
            )  # noqa: E501

        self._reason = reason

    @property
    def validation_errors(self):
        """Gets the validation_errors of this RelayerApiErrorResponseSchema.  # noqa: E501


        :return: The validation_errors of this RelayerApiErrorResponseSchema.  # noqa: E501
        :rtype: list[RelayerApiErrorResponseSchemaValidationErrors]
        """
        return self._validation_errors

    @validation_errors.setter
    def validation_errors(self, validation_errors):
        """Sets the validation_errors of this RelayerApiErrorResponseSchema.


        :param validation_errors: The validation_errors of this RelayerApiErrorResponseSchema.  # noqa: E501
        :type: list[RelayerApiErrorResponseSchemaValidationErrors]
        """

        self._validation_errors = validation_errors

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(
                    map(
                        lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                        value,
                    )
                )
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(
                    map(
                        lambda item: (item[0], item[1].to_dict())
                        if hasattr(item[1], "to_dict")
                        else item,
                        value.items(),
                    )
                )
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, RelayerApiErrorResponseSchema):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other
